#include <Wire.h>
#include <Adafruit_NeoPixel.h>

// Définition pour les leds
#define PIN_LED 3     // (numéroté 2 ou PB3 sur didigispark) Control signal: DI 
#define NUM_LED 3     // Number of LEDs in a strip
// Custom colour4: White
#define RED_VAL       255
#define GREEN_VAL     255
#define BLUE_VAL      255

// définitions pour l'ADXL345
#define ADXL345 (0x53) //address of Accelerometer
#define ADXL345_X (0x32) //register for X value from Accelerometer
#define ADXL345_Y (0x34) //register for Y value from Accelerometer
#define ADXL345_Z (0x36) //register for Z value from Accelerometer

// délai entre deux boucles 
#define DELAY 125 // à modifier si changement de couleurs trop ou pas assez rapide

Adafruit_NeoPixel RGB_Strip = Adafruit_NeoPixel(NUM_LED, PIN_LED, NEO_GRB + NEO_KHZ800);

byte _buff[6];
float X_out, Y_out, Z_out;  // valeur de l'adxl345
char POWER_CTL = 0x2D;  //Power Control Register
char DATA_FORMAT = 0x31;

void setup()
{
  // Préparation de l'ADXL345
  Wire.begin();
  initAccelerometer();

  // préparation des led 
  RGB_Strip.begin();
  RGB_Strip.show();
  RGB_Strip.setBrightness(128);    // Set brightness, 0-255 (darkest - brightest)

  // Cycle de couleurs pour montrer l'initialisation
  // init: couleur rouge
    colorWipe(RGB_Strip.Color(RED_VAL, GREEN_VAL*0, BLUE_VAL*0), DELAY);
  // delay(200);
  delay (500);
  // retour d'init: couleur vert
    colorWipe(RGB_Strip.Color(RED_VAL*0, GREEN_VAL, BLUE_VAL*0), DELAY);

}

void loop()
{
  /* 
   *  le mouvement de l'adxl va modifier les couleurs des leds:
  */
  // === lecture des données de l'Adxl === //
  readAccelerometer(ADXL345_X, _buff);
  X_out = (((int)_buff[1]) << 8) | _buff[0] ;
  X_out = abs(X_out)/256;

  Y_out = (((int)_buff[3]) << 8) | _buff[2] ;
  Y_out = abs(Y_out)/256;

  Z_out = (((int)_buff[5]) << 8) | _buff[4] ;
  Z_out = abs(Z_out)/256;
  colorWipe(RGB_Strip.Color(RED_VAL*X_out, GREEN_VAL*Y_out, BLUE_VAL*Z_out), DELAY);

  // delay(DELAY); // peut etre pas necessaire => cf. dernier param de colorWipe

}

void initAccelerometer(){

  // writeRegister(ADXL345, POWER_CTL, 0x08);
  writeRegister(ADXL345, DATA_FORMAT, 0x01);//set range to +/-4G
  writeRegister(ADXL345, POWER_CTL, 0x08); //power up and enable measurements
}

void writeRegister(uint8_t address, uint8_t reg, uint8_t val){

  Wire.beginTransmission(address);
  Wire.write(reg);
  Wire.write(val);
  Wire.endTransmission();

}

void readAccelerometer(byte address, byte _buff[]){

  Wire.beginTransmission(ADXL345); // start transmission to device
  Wire.write(address);     
  Wire.endTransmission();         // end transmission

  Wire.beginTransmission(ADXL345); // start transmission to device
  Wire.requestFrom(ADXL345, 6);    // request 6 bytes from device

  int i = 0;
  while (Wire.available())
  {
    _buff[i] = Wire.read();    // receive a byte
    i++;
  }

  Wire.endTransmission();         // end transmission

}

uint8_t readRegister(uint8_t address, uint8_t reg){

  Wire.beginTransmission(address);
  Wire.write(reg);
  Wire.endTransmission();
  Wire.requestFrom(address, (uint8_t)0x01);

  return Wire.read();

}


// Fill the dots one after the other with a color
void colorWipe(uint32_t c, uint16_t wait) {
    RGB_Strip.setPixelColor(0, c);
    RGB_Strip.setPixelColor(1, c);
    RGB_Strip.setPixelColor(2, c);
    RGB_Strip.show();
    delay(wait);
}
